#!/bin/sh
# chmod 755 avr.sh
# Usage: avr.sh [ OPTION ] [ & ] [ disown ]
# -----------------------------------------------------------------------------------
# Ermittelt Thermal-Werte der Disk-/Rack-Station und schreibt diese in eine Logdatei
# Codierung: CP65001 - UTF-8 mit / ohne '(with BOM)'
# For Windows use the Font: MS LineDraw -> https://www.fontpalace.com/font-download/MS+LineDraw/
# -----------------------------------------------------------------------------------
# Voraussetzung:
# Der SNMP-Dienst (SNMPv1, SNMPv2c) muss aktiviert und ein Community-String vergeben sein.
# -----------------------------------------------------------------------------------
# 
# cat /dev/zero > /dev/null
# -----------------------------------------------------------------------------------
AvrVersion="01.02.05"
## --- Get working Dir
workingDir=$(dirname "$(readlink -e "$0")")"/"
## --- Include AVR-Libery
. $workingDir"lib/lib.sh"
. $workingDir"lib/sensors.sh"
. $workingDir"lib/ansi.sh"
. $workingDir"lib/init.sh"
. $workingDir"lib/svg.sh"

set -o nounset  # Exit script if try to use an uninitialised variable

if [ "$(ps -o stat= -p $$)" == "S+" ]
then
#   "AvrLogger runs in the foreground"
    avrStartMode="TERMINAL"
    avrStatus=0
else
#   "AvrLogger runs in the background"
    avrStatus=1
    avrStartMode="DAEMON"
fi

trap STRG_C SIGINT SIGTERM

avrDebug=0  # DEBUG = Debug-Modus
avrInfo=0   # Echo AvrLogger & System Parameters
avrTest=0   # Test run with data logging
avrSVG=0

# ----------------------------------------------------------
# --- Check if an instance of AvrLogger is already running
# ----------------------------------------------------------
avrCheckPid

while (( "$#" )); do
    case $1 in
    	DEBUG)
            avrDebug=1
	    ;;
    	INFO)
            avrInfo=1
	    ;;
	    TEST)
		    avrTest=1
		    if [ "$#" -gt 1 ]
		        then AvrTick=$2; shift
		        else AvrTick=10
		    fi
	    ;;
	    SVG)
	        if [ $avrExecution -eq 0 ]; then avrSVG=1; fi
	    ;;
    esac
    shift 
done

LOGFILE=$workingDir"log/messages"

if [ $avrStatus -eq 1 ]; then
    if [ $avrExecution -eq 0 ]
    then
        # if we running in the background, disable DEBUG, INFO, SVG & TEST
        avrDebug=0; avrInfo=0; avrTest=0; avrSvg=0
        # Redirect STDERR and STDOUT to $LOGFILE
        exec >> $LOGFILE 2>&1
    else
        echo
        echo -e $escInv"AVR: An instance of AvrLogger is already running."$escRst
        echo -e $escInv"AVR: The execution of AvrLogger has been aborted."$escRst
        echo -e $escInv"AVR:         Press 'Return' to continue.         "$escRst
        exit
    fi
fi

if [[ $avrExecution -eq 1 && $avrTest -eq 0 ]]; then
    avrTest=1
    AvrTick=10
fi

# ----------------------------------------------------------
# --- AvrLogger initialize, set default Values & read avr.conf
# ----------------------------------------------------------
TimerStart
avrInit

TimerStart
checkSensors
TimerStop
timeSensor="$timerLap"

# ----------------------------------------------------------
# --- Print Terminal-Mask
# ----------------------------------------------------------
escMask
# --- Set Tick-Time to reduce CPU-Usage
if [ $AvrTick -lt 10 ]
    then Tick="0.25"
    else Tick="0.50"
fi

# ----------------------------------------------------------
# --- Composing headers (column names)
# --- !! Do not change the order !!!
# ----------------------------------------------------------
TabD=0; hLine=$(GetValue $AvrConf AvrHeadDate)
TabT=1; hLine+=$ColSign$(GetValue $AvrConf AvrHeadTime)
TabL=2; hLine+=$ColSign$(GetValue $AvrConf AvrHeadLoad)
TabC=3; hLine+=$ColSign$(GetValue $AvrConf AvrHeadCpu)
TabS=4; hLine+=$ColSign$(GetValue $AvrConf AvrHeadStatus)
TabA=5; hLine+=$ColSign$(GetValue $AvrConf AvrHeadRoom)      # Set to Zero if not avable
if [ $AvrEnv -eq 0 ]; then TabA=0; fi
TabH=6; hLine+=$ColSign$(GetValue $AvrConf AvrHeadHdd)
TabP=7; hLine+=$ColSign$(GetValue $AvrConf AvrHeadPwm)       # Set to Zero if not avable
if [ $fan_count -eq 0 ]; then TabP=0; fi
TabI=8; hLine+=$ColSign$(GetValue $AvrConf AvrHeadi2c)       # Set to Zero if not avable
if [ $i2c_count -eq 0 ]; then TabI=0; fi
TabLast=8

# ----------------------------------------------------------
# --- If AvrMode = SVG create SVG-Grafic and terminate
# ----------------------------------------------------------
if [[ $avrSVG -eq 1 && $avrStatus -eq 0 ]]; then
    SvgInit "10"
    initPlotData
    SvgInitPlot
    Svg 0

    TimerStop
    escEcho "AVR: Ready and terminated" "$timerLap"
    escEcho "----------------------------------------------------------------"
    exit
fi

# ----------------------------------------------------------
# --- Print AVR-Header to $LOGFILE
# ----------------------------------------------------------
initLog

# --- List Disk's
ListDisk
escEcho "SYS: Sensors scanned" "$timeSensor"

# ----------------------------------------------------------
# --- Read plot data and prepare
# --- If INFO-Mode don`t create SVG
# ----------------------------------------------------------
if [ $avrInfo -eq 0 ]; then
    SvgInit "10"
    initPlotData
    SvgInitPlot
    Svg 0
fi
TimerStop

if [ $avrExecution -eq 1 ]; then
    escEcho "AVR: An instance of AvrLogger is already running"
    sleep 2
    escEcho "AVR: Avr-Mode is restricted to 'TEST' & AvrTick = $AvrTick"
    sleep 2
fi

#############################################################
# At this Point: Exit AvrLogger if AvrMode was "INFO"
#############################################################
if [ $avrInfo -eq 1 ]; then exit 0; fi

if [ $avrExecution -eq 1 ]; then
    escEcho "AVR: Inizalization completed" "$timerLap"
    escEcho "----------------------------------------------------------------"
fi

# ----------------------------------------------------------
# --- Kopfzeile ausgeben (Spaltennamen für CSV
WriteHeadline "$hLine"

# ----------------------------------------------------------
# --- Aktuelles Datum merken für Daily-Log / SVG-Log
Rdate=$(date +%Y.%m.%d)
# ----------------------------------------------------------
# --- Haupt[endlos]schleife
# ----------------------------------------------------------
TickTock=$[$(date +%s)+1]
s_time=$[$(date +%s)+$SvgTick]
t_time=$[$(date +%s)+$AvrTick]

TimerStart
while [ true ]; do

    # ----------------------------------------------------------
    # --- Sensoren auslesen
    for (( m=1; m<=cpu_count; m++)) do
        read cpu_value[$m] < $Core'temp'${cpu_name[$m]}'_input'
    done
    
    for (( m=1; m<=fan_count; m++)) do
        read fan_value[$m] < $i2c'fan'${fan_name[$m]}'_input'
    done

    for (( m=1; m<=i2c_count; m++)) do
        read i2c_value[$m] < $i2c'temp'${i2c_name[$m]}'_input'
    done

    # ----------------------------------------------------------
	# --- MV-Array: Eine Reihe nach unter verschieben
    for ((i=1;i<=arr_row-1;i++)) do
	    for ((j=0;j<=arr_col;j++)) do
    	    arr[$i,$j]=${arr[$[i+1],$j]}
	    done
    done

    # --- CPU-Load / ssCpuIdle in Prozent
    # --- Die Abfrage mit "snmwalk" liefert Zeitnah das beste Ergebnis neben
    # --- allen anderen Systemfunktionen wie "top, uptime, loadavg" 
	arr[$arr_row,0]=$[100-$(snmpwalk -v 2c  -c $Com -O qv $Ip 1.3.6.1.4.1.2021.11.11.0)]

    # ----------------------------------------------------------
    # --- CPU-Termalmittelwert bilden
	mx=0
    for (( m=1; m<=cpu_count; m++)) do
    	mx=$[mx+cpu_value[$m]/100]
    done
	arr[$arr_row,1]=$[mx/(m-1)]
	
    # --- Disk/Rack-Station Thermal-Status  
	arr[$arr_row,2]=$[$(snmpwalk -v 2c  -c $Com -O qv $Ip 1.3.6.1.4.1.6574.1.2.0)*10]

    # --- # --- Environmental sensor
	if [ $AvrEnv -ne 0 ]
	    then arr[$arr_row,3]=$(snmpwalk -v 2c  -c $Com -O qv $AvrEnvIp $AvrEnvOID)
        else arr[$arr_row,3]=0
    fi

    # ----------------------------------------------------------
    # --- Min / Max Disk-Temperature
    # --- Die Abfrag per "snmpwalk" liefert ein Zeitverzögertes Ergebnis,
    # --- ist aber deutlich ressourcenschonender als über "/usr/bin/smartctl"
    # --- Daher kann über "avr.conf" entschieden werde was genutzt wird.
	if [ $UseSmart -eq 1 ]
		then read -r -a hdd <<< $(getSMART Temperature_Celsius)
		else hdd=($(snmpwalk -v 2c  -c $Com -O qv $Ip 1.3.6.1.4.1.6574.2.1.1.6))
	fi

	mx=${hdd[0]}
	mn=${hdd[0]}
	for(( m=1; m<${#hdd[@]}; m++)) do
		# --- Max
		if [ $mx -le ${hdd[$m]} ]
		    then mx=${hdd[$m]}
		fi
		# --- Min
		if [ $mn -gt ${hdd[$m]} ]
		    then mn=${hdd[$m]}
		fi
	done

    arr[$arr_row,4]=$[mx*10]

    # ----------------------------------------------------------
    # --- FAN-Values (PWM-Values) in percent (%)
	if [ $fan_count -gt 0 ]
	then
		mx=0
	   	for (( m=1; m<=fan_count; m++)) do
        	mx=$[mx+fan_value[$m]]
    	done
		mx=$[mx*100/(m-1)/Rpm]
        # --- intercept rounding error ---
		if [ $mx -gt 100 ]
		    then arr[$arr_row,5]=100
		    else arr[$arr_row,5]=$mx
	    fi
    else
        arr[$arr_row,5]=0
	fi

    # ----------------------------------------------------------
    # --- i2c Form and average the average temperature
	if [ $i2c_count -gt 0 ]; then
	    mx=0
        for (( m=1; m<=i2c_count; m++)) do
    	    mx=$[mx+i2c_value[$m]/100]
        done
	    arr[$arr_row,6]=$[mx/(m-1)]
    else
        arr[$arr_row,6]=0
    fi

    # ----------------------------------------------------------
    # --- Initial Moving Average Filter fill with the first values
	if [ $arr_cnt -eq 0 ]; then
		arr_cnt=1
        for ((i=0;i<=arr_row-1;i++)) do
	        for ((j=0;j<=arr_col;j++)) do
    	        arr[$i,$j]=${arr[$arr_row,$j]}
	        done
        done
	fi

    # ----------------------------------------------------------
    # --- Calculate Moving Average Filter
	for ((j=0;j<=arr_col;j++)) do
		arr[0,$j]=0
    	for ((i=1;i<=arr_row;i++)) do
        	arr[0,$j]=$((${arr[0,$j]}+${arr[$i,$j]}))
        done
        arr[0,$j]=$((${arr[0,$j]}/$arr_row))
	done

    # ----------------------------------------------------------
    # --- Output values
    # ----------------------------------------------------------
    line=$(date '+%Y.%m.%d;%H:%M:%S')

    # --- CPU-Load / ssCpuIdle in percent (%)
    line+=$ColSign${arr[0,0]}
    pine=" "$[arr[0,0]*10]

    # --- Standardize CPU temperatures
    line+=$(prt_2d1 ${arr[0,1]})
    pine+=" "${arr[0,1]}

    # --- Disk/Rack-Station Thermal-Status  
	line+=$(prt_2d1 ${arr[0,2]})
    pine+=" "${arr[0,2]}

    # --- Environmental sensor with 10 scales
	line+=$(prt_2d1 ${arr[0,3]})
    pine+=" "${arr[0,3]}

    # --- Highest Disk temperature
	line+=$(prt_2d1 ${arr[0,4]})
    pine+=" "${arr[0,4]}

    # --- Fan-Value or PWM-Value in percent (%)
   	line+=$ColSign${arr[0,5]}
   	pine+=" "$[arr[0,5]*10]

    # --- i2c Termal-Value
    line+=$(prt_2d1 ${arr[0,6]})
    pine+=" "${arr[0,6]}

    if [[ $AvrLogOn -eq 1 || $avrTest -eq 1 ]]; then
        if [ "$AvrLogMode" != "Free" ]
            then addLine "$line" "$AvrLog" 2 1440
            else addLine "$line" "$AvrLog" 0 1440
        fi
    fi
    addLine "$(date '+%Y.%m.%d %H:%M')$pine" "$AvrDat" 1 1440
    addPlotData "$(date '+%Y.%m.%d %H:%M')$pine"

    if [ $avrStatus -eq 0 ]; then
        TimerStop
        escData "$pine"
        TimerStart
    fi

	until [ $(date +%s) -ge $t_time ]; do
    
        if [[ $avrStatus -eq 0 && $(date +%s) -ge $TickTock ]]; then
            TickTock=$[$(date +%s)+1]
            escPrint 37 3 "$(printf "%3d" $[s_time-$(date +%s)])"
            escPrint 17 3 "$(printf "%3d" $[t_time-$(date +%s)])"
        fi   

        if [ $(date +%s) -ge $s_time ]; then
            s_time=$[$(date +%s)+$SvgTick]
            Svg $addNewLines
            addNewLines=0
        fi
        sleep "$Tick"
	done

    t_time=$[$(date +%s)+$AvrTick]

    # --- Archiving only if AvrLogger is running in the background
    if [[ $avrStatus -eq 1 && "$Rdate" != "$(date +%Y.%m.%d)" ]]; then

        if [[ $SvgOn -eq 1 && $SvgArchive -eq 1 ]]; then
            mv $SvgFile $workingDir"data/archive/"$Rdate" avr.svg"
            escEcho "AVR: SVG graphic has been archived"
        fi

        if [ $AvrLogOn -eq 1 ]; then
            if [ $AvrArchive -eq 1 ]; then
                mv $AvrLog $workingDir"data/archive/"$Rdate" avr.csv"
                escEcho "AVR: CSV sheet has been archived"
            fi
            if [ "$AvrLogMode" == "Daily" ]; then WriteHeadline "$hLine"; fi
        fi
        Rdate=$(date +%Y.%m.%d)
    fi
done

# -------- END --------
