SvgVersion="01.01.03"
SvgEngine="AvrLogger v"$AvrVersion" SVG-Engine v"$SvgVersion", : https://www.luenepiet.de/my_avrlogger/main"
SvgPageID="AVRlog"

# ----------------------------------------------------------
# --- Prepare SVG-Values
# --- $1 = ViewBoxScale
# ----------------------------------------------------------

SvgInit()
{
    PageSizeWidht=$(GetValue  $SvgConf PageSizeWidht)
    PageSizeHeight=$(GetValue $SvgConf PageSizeHeight)
    ViewBoxWidth=$[PageSizeWidht*$1]
    ViewBoxHeight=$[PageSizeHeight*$1]

    BorderEdge=$(GetValue $SvgConf BorderEdge)
    BorderX=$BorderEdge
    BorderY=$BorderEdge
    BorderWidth=$[ViewBoxWidth-(2*BorderEdge)]
    BorderHeight=$[ViewBoxHeight-(2*BorderEdge)]
    # --- Plotting Area ---
    PlotAreaFill=$(GetValue   $SvgConf PlotAreaFill)
    # ----------------------------------------
    # --- Prepare Borders
    # ----------------------------------------
    PlotEdgeLeft=$(GetValue   $SvgConf "PlotEdgeLeft")
    PlotEdgeRight=$(GetValue  $SvgConf "PlotEdgeRight")
    PlotEdgeTop=$(GetValue    $SvgConf "PlotEdgeTop")
    PlotEdgeBottom=$(GetValue $SvgConf "PlotEdgeBottom")

    PlotLabelWidth=$(GetValue $SvgConf  "PlotLabelWidth")
    PlotLabelHeight=$(GetValue $SvgConf "PlotLabelHeight")

    pw_l=0; pw_r=0; pw_v=0
    if [ "$(GetValue $SvgConf "PriLabelOn")" == "Yes" ]; then PlotEdgeLeft=$[PlotEdgeLeft+PlotLabelWidth]; pw_l=PlotLabelWidth; fi
    if [ "$(GetValue $SvgConf "SecLabelOn")" == "Yes" ]; then PlotEdgeRight=$[PlotEdgeRight+PlotLabelWidth]; pw_r=PlotLabelWidth; fi
    if [ "$(GetValue $SvgConf "VerLabelOn")" == "Yes" ]; then PlotEdgeBottom=$[PlotEdgeBottom+PlotLabelHeight]; pw_v=PlotLabelHeight; fi

    LegAlignment=$(GetValue $SvgConf   "LegAlignment") # Orientation "Hor" (horizontal) "Ver" (vertical)
    LegZeroX=$(GetValue $SvgConf       "LegZeroX")     # X-Zero "Left" / "Right"
    LegZeroY=$(GetValue $SvgConf       "LegZeroY")     # Y-Zero "Top" / "Bottom"
    LegOffsetX=$(GetValue $SvgConf     "LegOffsetX")
    LegOffsetY=$(GetValue $SvgConf     "LegOffestY")
    LegTabX=$(GetValue $SvgConf        "LegTabX")
    LegTabY=$(GetValue $SvgConf        "LegTabY")

    if [ "$(GetValue $SvgConf "LegLabelOn")" == "Yes" ]
    then
        if [[ "$LegZeroY" == "Top"    && "$LegAlignment" == "Hor" ]]; then PlotEdgeTop=$[PlotEdgeTop+LegTabY+(LegOffsetY/2)]; fi
        if [[ "$LegZeroY" == "Bottom" && "$LegAlignment" == "Hor" ]]; then PlotEdgeBottom=$[PlotEdgeBottom+LegTabY+(2*LegOffsetY)]; fi
        if [[ "$LegZeroX" == "Left"   && "$LegAlignment" == "Ver" ]]; then PlotEdgeLeft=$[PlotEdgeLeft+LegTabX+(2*LegOffsetX)]; fi
        if [[ "$LegZeroX" == "Right"  && "$LegAlignment" == "Ver" ]]; then PlotEdgeRight=$[PlotEdgeRight+LegTabX+(2*LegOffsetX)]; fi
        if [  "$LegZeroX" == "Left" ]; then LegLabelAnchor="start"; fi
        if [  "$LegZeroX" == "Right" ]; then LegLabelAnchor="end"; fi
    else
        LegLabelAnchor="start"
    fi

    pw=$[ViewBoxWidth-PlotEdgeLeft-PlotEdgeRight]
    ph=$[ViewBoxHeight-PlotEdgeTop-PlotEdgeBottom]
}

Svg()
{
    if [ $SvgOn -eq 0 ]; then return; fi

    # --- Path to SVG-Plot
    AvrSvg=$workingDir"tmp/tmp.svg"
    SvgGraph=$workingDir"tmp/graph.tmp"
    TimerStart
    # ----------------------------------------
    # --- Prepare SVG
    # ----------------------------------------
    # --- Copy SvgGraph-Template to AvrSbg and overwrite earlier File
    cp "$SvgGraph" "$AvrSvg"

    SvgLabel "Ver" "Verticaly" "v"
    echo -e '\t\t</g> <!-- VerticalAxisEnd -->' >> $AvrSvg
    # ----------------------------------------

    Stroke=$(GetValue   $SvgConf PlotLineStroke)
    x=$PlotEdgeLeft
    y=$[($PlotEdgeTop+ph)*-1]
    echo -e '\t\t<g id="DataPlots" transform="scale(1,-1) translate('$x','$y')" fill="none" stroke-width="'$Stroke'">'  >> $AvrSvg

    # ----------------------------------------
    SvgPlot "Load"     $TabL
    SvgPlot "CPU"      $TabC
    SvgPlot "Status"   $TabS
    SvgPlot "Ambiente" $TabA
    SvgPlot "Hdd"      $TabH
    SvgPlot "Pwm"      $TabP
    SvgPlot "i2c"      $TabI
    # ----------------------------------------

    echo -e '\t\t</g> <!-- DataPlotsEnd -->' >> $AvrSvg
    # ----------------------------------------
    echo -e "\t</g> <!-- PageEnd -->" >> $AvrSvg
    echo "</svg> <!-- AvrLoggerEnd -->" >> $AvrSvg

	chmod 666 $AvrSvg

    # --- Copy "Tempfile" to destination and overwrite earlier File
    cp "$AvrSvg" "$SvgFile"

    # --- Copy SVG-File to users defined destination
    if [ $avrTest -eq 0 ]
        then SvgCopy=$(GetValue $AvrConf SvgCopy)
        else SvgCopy=$(GetValue $AvrConf SvgCopyTest)
    fi
    if [ ${#SvgCopy} -gt 0 ]; then
        # --- Copy "SVG-File" to destination and overwrite earlier File
        cp "$AvrSvg"  "$SvgCopy"
    fi

    TimerStop
    if [ $1 -eq 0 ]
        then escEcho 'SVG: SVG graphic created' "$timerLap"
        else escEcho 'SVG: SVG graphics updated with -'$1'- new values' "$timerLap"
    fi
}

# ----------------------------------------------------------
# --- Prepare SVG-Header
# --- $1 = ViewBoxScale
# ----------------------------------------------------------
SvgHeader()
{
    TimerStart
    # --- Main Values
    PutSvg "$AvrSvg" "SvgEngine" "$SvgEngine"
    PutSvg "$AvrSvg" "SvgPageID" "$SvgPageID"

    PutSvg "$AvrSvg" "PageSizeWidht"  "$PageSizeWidht"
    PutSvg "$AvrSvg" "PageSizeHeight" "$PageSizeHeight"
    PutSvg "$AvrSvg" "ViewBoxWidth"  "$ViewBoxWidth"
    PutSvg "$AvrSvg" "ViewBoxHeight" "$ViewBoxHeight"

    # --- Boarder Values
    CpySvg "$SvgConf" "$AvrSvg" "BorderColor"
    CpySvg "$SvgConf" "$AvrSvg" "BorderStroke"
    CpySvg "$SvgConf" "$AvrSvg" "BorderFill"
    PutSvg "$AvrSvg" "BorderX"      "$BorderX"
    PutSvg "$AvrSvg" "BorderY"      "$BorderY"
    PutSvg "$AvrSvg" "BorderWidth"  "$BorderWidth"
    PutSvg "$AvrSvg" "BorderHeight" "$BorderHeight"

    if [ "$PlotAreaFill" != "none" ]; then
        x1=$PlotEdgeLeft
        y1=$PlotEdgeTop
        x2=$[ViewBoxWidth-PlotEdgeLeft-PlotEdgeRight]
        y2=$[ViewBoxHeight-PlotEdgeTop-PlotEdgeBottom]

        echo -e '\t\t<g id="PlotFrame" fill="'$PlotAreaFill'" stroke="none" stroke-width="1">' >> $AvrSvg
        echo -e '\t\t\t<rect x="'$x1'" y="'$y1'" width="'$x2'" height="'$y2'"/>' >> $AvrSvg
        echo -e '\t\t</g> <!-- PlotFrameEnd -->' >> $AvrSvg
    fi
    # ----------------------------------------
    # --- Legende einfügen
    # ----------------------------------------
    Font=$(GetValue $SvgConf   "LegLabelFont")
    Size=$(GetValue $SvgConf   "LegLabelSize")
    echo -e '\t\t<g id="Legend" text-anchor="'$LegLabelAnchor'" font-family="'$Font'" font-size="'$Size'" stroke-width="1">' >> $AvrSvg
    # ----------------------------------------
    Count=0

    SvgLegend "Load"     $TabL
    SvgLegend "CPU"      $TabC
    SvgLegend "Status"   $TabS
    SvgLegend "Ambiente" $TabA
    SvgLegend "Hdd"      $TabH
    SvgLegend "Pwm"      $TabP
    SvgLegend "i2c"      $TabI

    echo -e '\t\t</g> <!-- LegendEnd -->' >> $AvrSvg

    TimerStop
    escEcho 'SVG: Header created' "$timerLap"
}

# ----------------------------------------------------------
# --- Draw Grid
# --- $1 = First Name
# --- $2 = Second Name
# --- $3 = Alignment
# ----------------------------------------------------------
SvgGrid()
{
    if [ "`GetValue $SvgConf "$1$2GridOn"`" != "Yes" ]; then return; fi

    Min=$(GetValue $SvgConf    "$1Min")
    Max=$(GetValue $SvgConf    "$1Max")
    Step=$(GetValue $SvgConf   "$1$2Interval")
    Main=$(GetValue $SvgConf   "$1MainInterval")
    Color=$(GetValue $SvgConf  "$1$2GridColor")
    Stroke=$(GetValue $SvgConf "$1$2GridStroke")
    Off1=$(GetValue $SvgConf   "$1$2GridOff1")
    Off2=$(GetValue $SvgConf   "$1$2GridOff2")

    Range=$[Max-Min]

    if [ "$3" == "h" ]
        then
            Inc=$[(ph*10)/(Range/Step)]
            h=$[Off1+pw+Off2]
            x=$[PlotEdgeLeft-Off1]
            yz=$[(PlotEdgeTop+ph)*10]
        else
            Inc=$[(pw*10)/(Range/Step)]
            h=$[Off1+ph+Off2]
            y=$[PlotEdgeTop-Off1]
            xz=$[PlotEdgeLeft*10]
    fi

    c=0; a=0; b=0
    e_line='\t\t\t<g id="'$1$2'Grid" fill="none" stroke-width="'$Stroke'" stroke="'$Color'">\n'
    for (( i=$Min; i <= $Max; i=i+$Step)) do
        if [[ $a -eq $b && "$2" == "Help" ]]
            then
                a=$[a+$Main]
            else
                if [ "$3" == "h" ]
                    then y=$[(yz-(Inc*c))/10]
                    else x=$[(xz+(Inc*c))/10]
                fi
                e_line+='\t\t\t\t<path d="M'$x','$y' '$3$h'"/>\n'
        fi
        b=$[b+$Step]
        c=$[c+1]
    done
    echo -e "$e_line"'\t\t\t</g> <!-- '$1$2'GridEnd -->' >> $AvrSvg

    h=$[Off1+ph+Off2]
    y=$[PlotEdgeTop-Off1]
    xz=$[PlotEdgeLeft*10]
}

# ----------------------------------------------------------
# --- Print Grid Labels
# --- $1 = Name
# --- $2 = ID
# --- $3 = Alignment
# ----------------------------------------------------------
SvgLabel()
{
    if [ "`GetValue $SvgConf "$1LabelOn"`" != "Yes" ]; then return; fi

    Min=$(GetValue $SvgConf    "$1Min")
    Max=$(GetValue $SvgConf    "$1Max")
    Step=$(GetValue $SvgConf   "$1MainInterval")
    Font=$(GetValue $SvgConf   "$1LabelFont")
    Size=$(GetValue $SvgConf   "$1LabelSize")
    Pos=$(GetValue $SvgConf    "$1LabelPos")
    Offset=$(GetValue $SvgConf "$1LabelOffset")
    Anchor=$(GetValue $SvgConf "$1LabelAnchor")
    Suffix=$(GetValue $SvgConf "$1LabelSuffix")

    Range=$[Max-Min]

    l_svg='\t\t\t<g id="'$2'Label" text-anchor="'$Anchor'" font-family="'$Font'" font-size="'$Size'">\n'

    if [ "$3" == "h" ]
        then
            if [ "$1" == "Pri" ]
	            then x=$[PlotEdgeLeft-Pos]
	            else x=$[PlotEdgeLeft+pw+Pos]
    	    fi
            Inc=$[(ph*10)/(Range/Step)]
            yz=$[(PlotEdgeTop+ph)*10]
        else
            Inc=$[(pw*10)/(Range/Step)]
        	y=$[PlotEdgeTop+ph+Pos]
            xz=$[PlotEdgeLeft*10]
    fi

    c=0
    for (( i=Min; i <= Max; i=i+Step)) do
        if [ "$3" == "h" ]
            then
                y=$[((yz-(Inc*c))/10)+Offset]
		        l_svg+='\t\t\t\t<text x="'$x'" y="'$y'">'$i' '$Suffix'</text>\n'
            else
            # --- Vertikale Achsenbeschriftung
                t=$[svgTimeline+(c*(Step*AvrTick))]
                if [ $t -ge 86400 ]; then
                    t=$[t-86400]
                fi
                hh=$[t/3600]
                mm=$[(t-(hh*3600))/60]
                t="$(printf "%02d" $hh):$(printf "%02d" $mm)"
                x=$[((xz+(Inc*c))/10)-Offset]
		        l_svg+='\t\t\t\t<text x="'$x'" y="'$y'">'$t$Suffix'</text>\n'
        fi
        c=$[c+1]
    done
    echo -e $l_svg'\t\t\t</g> <!-- '$2'LabelEnd -->' >> $AvrSvg
}

# ----------------------------------------------------------
# --- Create Legend of Plot-Sensors
# --- $1 = Plot Name
# --- $2 = Plot ID
# ----------------------------------------------------------
SvgLegend()
{
    if [ $2 -eq 0 ]; then return; fi
    if [ "`GetValue $SvgConf "LegLabelOn"`" != "Yes" ]; then return; fi
    if [ "`GetValue $SvgConf 'Plot'$1'On'`" != "Yes" ]; then return; fi

    Color=`GetValue $SvgConf 'Plot'$1'Color'`
    echo -e '\t\t\t<g id="Legend'$1'" fill="'$Color'" stroke="'$Color'">' >> $AvrSvg

    # ------------------------------
    if [ "$LegAlignment" == "Hor" ]
        then LegendTab_y=0;        LegendTab_x=$LegTabX;
        else LegendTab_y=$LegTabY; LegendTab_x=0
    fi
    # ------------------------------
    if [ "$LegZeroX" == "Left" ]
    then
        if [ "$LegAlignment" == "Hor" ]
            then LegendZero_x=$[PlotEdgeLeft + LegOffsetX]
            else LegendZero_x=$[PlotEdgeLeft - LegTabX - LegOffsetX - pw_l]
        fi
    else
        LegendTab_x=$[LegendTab_x * -1]
        if [ "$LegAlignment" == "Hor" ]
            then LegendZero_x=$[PlotEdgeLeft + pw - LegTabX - LegOffsetX]
            else LegendZero_x=$[PlotEdgeLeft + pw  + pw_r + LegOffsetX]
        fi
    fi
    # ------------------------------
    if [ "$LegZeroY" == "Top" ]
    then
        if [ "$LegAlignment" == "Hor" ]
            then LegendZero_y=$[PlotEdgeTop - (LegTabY / 2) - LegOffsetY]
            else LegendZero_y=$[PlotEdgeTop + (LegTabY / 2) ] # + LegOffsetY]
        fi
    else
        LegendTab_y=$[LegendTab_y * -1]
        if [ "$LegAlignment" == "Hor" ]
            then LegendZero_y=$[PlotEdgeTop + ph + (LegTabY / 2) + (LegOffsetY * 2) + pw_v]
            else LegendZero_y=$[PlotEdgeTop + ph - (LegTabY / 2) ] # - LegOffsetY]
        fi
    fi
    # ------------------------------

    x=$[LegendZero_x + ( Count * LegendTab_x )]
    y=$[LegendZero_y + ( Count * LegendTab_y )]

    # Signs: Line / Circle / Square
    Sign=$(GetValue $SvgConf "LegSign")
    Size=$(GetValue $SvgConf "LegSignSize")

    case $Sign in
    	Line)
            h=20; w=$Size
            if [ "$LegZeroX" == "Left" ]
            then
                x1=$x;                y1=$[y-(h/2)]
            else
                x1=$[x+LegTabX-Size]; y1=$[y-(h/2)]
            fi
            echo -e '\t\t\t\t<rect x="'$x1'" y="'$y1'" width="'$w'" height="'$h'"/>' >> $AvrSvg
	    ;;
	    Circle)
            h=$[Size/2]
            if [ "$LegZeroX" == "Left" ]
            then
                x1=$[x+h];         y1=$y
            else
                x1=$[x+LegTabX-h]; y1=$y
            fi                
            echo -e '\t\t\t\t<circle cx="'$x1'" cy="'$y1'" r="'$h'"/>' >> $AvrSvg
	    ;;
	    Square)
            h=$Size; w=$Size
            if [ "$LegZeroX" == "Left" ]
            then
                x1=$x;             y1=$[y-(h/2)]
            else
                x1=$[x+LegTabX-h]; y1=$[y-(w/2)]
            fi
            echo -e '\t\t\t\t<rect x="'$x1'" y="'$y1'" width="'$w'" height="'$h'"/>' >> $AvrSvg
	    ;;
    esac

    Label=$(GetValue $SvgConf      'Plot'$1'Label')
    LabelColor=$(GetValue $SvgConf "LegLabelColor")
    Offset=$(GetValue $SvgConf     "LegLabelOffset")

    if [ "$LegZeroX" == "Left" ]
    then
        x1=$[x+Size+(Size/2)];         y1=$[y+Offset]
    else
        x1=$[x+LegTabX-Size-(Size/2)]; y1=$[y+Offset]
    fi
    
    echo -e '\t\t\t\t<text fill="'$LabelColor'" stroke="'$LabelColor'" x="'$x1'" y="'$y1'">'$Label'</text>' >> $AvrSvg
    echo -e '\t\t\t</g> <!-- Legend'$1'End -->' >> $AvrSvg

    Count=$[Count+1]

}

declare -a x_Plot

SvgInitPlot()
{
    TimerStart

    # --- Path to SVG-Plot
    AvrSvg=$workingDir"tmp/tmp.svg"
    SvgHeader=$workingDir"templates/header.svg"
    SvgGraph=$workingDir"tmp/graph.tmp"

    # ----------------------------------------
    # --- Prepare SVG-Header
    # ----------------------------------------
    # --- Copy "Header" to SvgGraph-Template and overwrite earlier File
    cp "$SvgHeader" "$SvgGraph"
    AvrSvg="$SvgGraph"
    SvgHeader

    # ----------------------------------------
    echo -e '\t\t<g id="PrimaryAxis">' >> $AvrSvg
    SvgGrid "Pri" "Main" "h"
    SvgGrid "Pri" "Help" "h"
    SvgLabel "Pri" "Primary" "h"
    echo -e '\t\t</g> <!-- PrimaryAxisEnd -->' >> $AvrSvg
    # ----------------------------------------
    echo -e '\t\t<g id="SecondaryAxis">' >> $AvrSvg
    SvgGrid "Sec" "Main" "h"
    SvgGrid "Sec" "Help" "h"
    SvgLabel "Sec" "Secondary" "h"
    echo -e '\t\t</g> <!-- SecondaryAxisEnd -->' >> $AvrSvg
    # ----------------------------------------
    # --- Vertical Axis
    # ----------------------------------------
    echo -e '\t\t<g id="VerticalAxis">' >> $AvrSvg
    SvgGrid "Ver" "Main" "v"
    SvgGrid "Ver" "Help" "v"
    # ----------------------------------------
    TimerStop
    escEcho 'SVG: Graph-Template created' "$timerLap"


}


# ----------------------------------------------------------
# --- Generate Plot-Template
# --- $1 = Name
# --- $2 = Number
# ----------------------------------------------------------
SvgPlot()
{
    if [[ $2 -eq 0 || $minRow -eq 0 || "$(GetValue $SvgConf 'Plot'$1'On')" != "Yes" ]]; then return; fi

    Color=$(GetValue $SvgConf 'Plot'$1'Color')
	echo -e '\t\t\t<polyline id="Plot'$1'" stroke="'$Color'" points="'  >> $AvrSvg

    c=0
    for ((i=0;i<=$csvRow;i++)) do
        if [ $c -eq 0 ]; then e_line='\t\t\t'; fi
        e_line+=${plotData[$i,0]}","${plotData[$i,$2]}

        if [[  $c -lt 9  &&  $i -le $csvRow ]]
            then e_line+=" "; c=$[c+1]
            else echo -n -e $e_line"\n" >> $AvrSvg; c=0
        fi
	done

    if [ $c -ne 0 ]; then echo -n -e '\n' >> $AvrSvg; fi
	echo -e '\t\t\t"/> <!-- Plot'$1'End -->' >> $AvrSvg
}

# ----------------------------------------------------------
# Print first and last 3 rows of the plotData-Array
# ----------------------------------------------------------
# > Is not needed for the program flow and can be removed <
# ----------------------------------------------------------
SvgListPart()
{

    temp=$[csvRow+1]
    echo "----------------------- Start -----------------------"
    echo " Numer of Points : "$temp
    for ((i=0;i<=2;i++)) do
		for ((j=0;j<=csvCol;j++)) do
	    	echo -n "${plotData[$i,$j]} "
		done
		echo
	done

    echo "  :"
    echo "  :"
    echo "  :"

    for ((i=(csvRow-2);i<=csvRow;i++)) do
		for ((j=0;j<=csvCol;j++)) do
	    	echo -n "${plotData[$i,$j]} "
		done
		echo
	done
    echo "----------------------- End ------------------------"
}

